package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;

// Spelchan Imports
import com.blazinggames.urcl.data.Config;
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.sprite.*;
import com.blazinggames.urcl.ui.playfield.*;
import com.blazinggames.urcl.ui.comp.*;

/**
 * Ultimate Retro Project Episode 2: Modern Doors
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * Runs the Modern Doors game.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class DoorsDisplay 
extends Display
{
// *********************
// ***** CONSTANTS *****
// *********************
    private static final Point DEF_PLACE = new Point (0,0);
    private static final Rectangle DEF_RECT = new Rectangle(0,0,0,0);

    // Configuration keys
    private static final String BACKDROP_IMAGE = "02BACKDROP";
    private static final String TILE_IMAGE = "02TILE";
    private static final String TILE_SIZE = "02TILESIZE";
    private static final String HALL_PLACEMENT = "02TILERECT";
    private static final String SCROLL_PLACEMENT = "02SCROLLRECT";
    private static final String NUM_LEVELS = "02LEVELS";
    private static final String LEVEL = "02LEVEL";
    private static final String HISTORY = "02HISTORY";
    private static final String FONT = "02FONT";
    private static final String ROUND_PLACE = "02ROUND";
    private static final String REMAINING_PLACE = "02REMAINING";
    private static final String CURRENT_GUESS_PLACE = "02CURRENTGUESS";
    private static final String LAST_GUESS_PLACE = "02LASTGUESS";
    private static final String LAST_DIRECTION_PLACE = "02LASTDIR";
    private static final String LOW_STRING = "02LOWSTRING";
    private static final String HIGH_STRING = "02HIGHSTRING";
    private static final String CORRECT_STRING = "02CORRECTSTRING";
    private static final String LOST_STRING = "02LOSTSTRING";
    private static final String QUIT_TEXT = "02QUITTEXT";
    private static final String QUIT_PLACEMENT = "02QUITPOS";
    private static final String FRIEND_SPRITE = "02SPRITE";
    private static final String FRIEND_SIZE = "02SPRITESIZE";
    private static final String FRIEND_ADJUST = "02SPRITEADJ";
    
    // Sound support
    private static final String USE_SOUND = "02USESOUND";
    private static final String SOUND_IMAGE = "02SOUNDIMAGE";
    private static final String SOUND_RECT = "02SOUNDRECT";
    private static final String DOOR_SOUND = "02DOORSOUND";
    private static final String WIN_SOUND = "02WINSOUND";
    private static final String LOSE_SOUND = "02LOSESOUND";
    
    private static final String TILE_KEY = "DOORTILES";
    
    // Information displaying sprite locations
    private static final int SPRITE_ROUND = 0;
    private static final int SPRITE_REMAINING = 1;
    private static final int SPRITE_CURRENT_GUESS = 2;
    private static final int SPRITE_LAST_GUESS = 3;
    private static final int SPRITE_LAST_DIRECTION = 4;


// *********************
// ***** VARIABLES *****
// *********************

    Image _background;
    java.util.Random _random;
    
    // sprites
    TextSprite _sprites[];
    ImageStripSprite _friend;
    Point _friendAdjust;
    
    // Playfield
    Playfield _hall;
    Rectangle _hallPlacement;
    Rectangle _hallViewport;
    ImageStripTile _doorTile;
    Dimension _tileSize;
    Rectangle _scrollbarRect;
    SScrollBar _scrollbar;
    Slider _slider;
    TextButton _quitButton;
    Sprite _historySprites[];
    
    // Game Levels
    Image _levelSBImages[];
    int _levelSBYOffset[];
    int _levelSBHeight[];
    int _levelNumDoors[];
    int _levelGuesses[];
    Rectangle _levelHistory[];
    
    // Direction strings
    String _lowGuess;
    String _highGuess;
    String _correctGuess;
    String _lostGuess;
    
    // Game management variables
    int _level;
    int _room;
    int _round;
    int _currentGuess;
    int _lastGuess;
    boolean _makingGuess;
    boolean _gameOver;
    int _tickCount;

    // Sound management
    boolean _useSound;
    Toggle _soundToggle;
    
// *****************************************
// ***** METHODS - Initialization aids *****
// *****************************************

    /**
     * prepares all the label fonts as well as loads in the friend image
    */
    private void prepareSprites()
    {
        Container c = _displayHandler.getContainer();
        Config cf = _displayHandler.getConfig();
                
        _sprites = new TextSprite[5];
        Point p = cf.getPointProperty(ROUND_PLACE, DEF_PLACE);
        _sprites[SPRITE_ROUND] = new TextSprite(c, "round", p.x, p.y);
        p = cf.getPointProperty(REMAINING_PLACE, DEF_PLACE);
        _sprites[SPRITE_REMAINING] = new TextSprite(c, "remaining", p.x, p.y);
        p = cf.getPointProperty(CURRENT_GUESS_PLACE, DEF_PLACE);
        _sprites[SPRITE_CURRENT_GUESS] = new TextSprite(c, "current", p.x, p.y);
        p = cf.getPointProperty(LAST_GUESS_PLACE, DEF_PLACE);
        _sprites[SPRITE_LAST_GUESS] = new TextSprite(c, "last", p.x, p.y);
        p = cf.getPointProperty(LAST_DIRECTION_PLACE, DEF_PLACE);
        _sprites[SPRITE_LAST_DIRECTION] = new TextSprite(c, "dir", p.x, p.y);

        font = cf.getFontProperty(FONT, font);
        for (int cntr = 0; cntr < 5; ++cntr)
        {
            _sprites[cntr].setFont(c, font);
            _sprites[cntr].setVisible(true);
            _sprites[cntr].setSolid(false);
        }
        
        _lowGuess = _config.getStringProperty(LOW_STRING, "low");
        _highGuess = _config.getStringProperty(HIGH_STRING, "high");
        _correctGuess = _config.getStringProperty(CORRECT_STRING, "correct");
        _lostGuess = _config.getStringProperty(LOST_STRING, "lost");
        _res.addImage(FRIEND_SPRITE, _config.getStringProperty(FRIEND_SPRITE, "friend.gif"));
        _friendAdjust = _config.getPointProperty(FRIEND_ADJUST, DEF_PLACE);
        p = _config.getPointProperty(FRIEND_SIZE, DEF_PLACE);
        _friend = new ImageStripSprite(c, _res.getImage(FRIEND_SPRITE), p.x, p.y);
        _friend.setVisible(true);
        _friend.setSolid(false);
    }
    
    /**
     * Prepares the hall, which is simply a playfield so it can easily be scrolled.
    */
    private void preparePlayfield()
    {
        Container c = _displayHandler.getContainer();
        _hall = new Playfield();
        _hallPlacement = _config.getRectangleProperty(HALL_PLACEMENT, DEF_RECT);
        _hallViewport = new Rectangle(0,0, _hallPlacement.width, _hallPlacement.height);
        Point p = _config.getPointProperty(TILE_SIZE, DEF_PLACE);
        _tileSize = new Dimension(p.x, p.y);
        _res.addImage(TILE_KEY, _config.getStringProperty(TILE_IMAGE, "doors.gif"));
        _doorTile = new ImageStripTile(_res.getImage(TILE_KEY), p.x, p.y);
        _hall.setTileset(2, p.x, p.y);
        _hall.setTileRenderer(0, _doorTile);
        _slider = new Slider();
        _slider.setSolid(false);
        Rectangle r = _config.getRectangleProperty(SCROLL_PLACEMENT, DEF_RECT);
        _scrollbarRect = r;
        _scrollbar = new SScrollBar(r.x, r.y, r.width, r.height, true);
        
        _quitButton = new TextButton(c,
                    _config.getStringProperty(QUIT_TEXT, "MENU"),
                    _config.getRectangleProperty(QUIT_PLACEMENT, DEF_RECT) );
    }

    private void prepareSounds()
    {
        _useSound = (_config.getIntProperty(USE_SOUND, 0) == 1);
        _res.addImage(SOUND_IMAGE, _config.getStringProperty(SOUND_IMAGE, "sound.gif"));
        Rectangle r = _config.getRectangleProperty(SOUND_RECT, DEF_RECT);
    	_soundToggle = new Toggle(_displayHandler.getContainer(),
                    _res.getImage(SOUND_IMAGE), r.x, r.y, r.width, r.height, 2);
        if (_useSound)
        {
            _res.addAudioClip(DOOR_SOUND, _config.getStringProperty(DOOR_SOUND, "silent.au"));
            _res.addAudioClip(WIN_SOUND, _config.getStringProperty(WIN_SOUND, "silent.au"));
            _res.addAudioClip(LOSE_SOUND, _config.getStringProperty(LOSE_SOUND, "silent.au"));
        }
    }

    /**
     * processes the levels and loads all the slider backdrops.
    */
    public void prepareLevels()
    {
        Container c = _displayHandler.getContainer();
        int n = _config.getIntProperty(NUM_LEVELS, 0);
        if (n < 1)
            n = 1;
        
        _levelSBImages = new Image[n];
    	_levelSBYOffset = new int[n];
    	_levelSBHeight = new int[n];
    	_levelNumDoors = new int[n];
    	_levelGuesses = new int[n];
        _levelHistory = new Rectangle[n];
        
        String args[];
        for (int cntr = 0; cntr < n; ++cntr)
        {
            args = _config.parseString(_config.getStringProperty(LEVEL+"["+cntr+"]","unnamed,100,7"),',');
            if (args.length != 5)
            {
                _levelSBImages[cntr] = c.createImage(1, 1);
                _levelSBYOffset[cntr] = 0;
                _levelSBHeight[cntr] = 50;
                _levelNumDoors[cntr] = 10;
                _levelGuesses[cntr] = 5;
            }
            else
            {
                _res.addImage(args[0], args[0]);
                _levelSBImages[cntr] = _res.getImage(args[0]);
                _levelSBYOffset[cntr] = _config.parseInt(args[1], 0);
                _levelSBHeight[cntr] = _config.parseInt(args[2], 50);
                _levelNumDoors[cntr] = _config.parseInt(args[3], 10);
                _levelGuesses[cntr] = _config.parseInt(args[4], 5);
            }
            _levelHistory[cntr] = _config.getRectangleProperty(HISTORY+"["+cntr+"]", DEF_RECT);
        }
    }


// ***********************************
// ***** METHODS - Game Handling *****
// ***********************************

    /**
     * Used by the controller applet/application to set the level to play.
     *
     * @param n level to play (0 being first level)
    */
    public void setLevel(int n)
    {
        _level = n;
        startNewGame();
    }
    
    /**
     * prepares for a new game
    */
    private void startNewGame()
    {
        int doors = _levelNumDoors[_level];
        _hall.setPlayfield(doors, 1);
        _hall.clear(0,0);
        _scrollbar.setBarImage(_levelSBImages[_level]);
        _slider.scale(10, _levelSBHeight[_level]);
        _scrollbar.setSliderType(_slider, 0, _levelSBYOffset[_level]);
        _scrollbar.setSlider(0);
        _scrollbar.setRange(0, doors*_tileSize.width, _hallPlacement.width);
        
        _room = ((_random.nextInt() & Integer.MAX_VALUE) % doors) + 1;
        _displayHandler.log("Room is " + _room);
        _round = 1;
        _currentGuess = 0;
        _lastGuess = 0;
        _gameOver = false;
        _makingGuess = false;
        
        _historySprites = new Sprite[doors];
        for (int cntr = 0; cntr < doors; ++cntr)
        {
            _historySprites[cntr] = new Sprite();
            _historySprites[cntr].setSolid(true);
            _historySprites[cntr].setBackground(Color.green);
        }
        updateLabels();
    }


    /**
     * Updates all the text sprites on the screen to reflect the current
     * game state.
    */
    private void updateLabels()
    {
        Container c = _displayHandler.getContainer();
        
        _sprites[SPRITE_ROUND].setText(c, "" + _round);
    	_sprites[SPRITE_REMAINING].setText(c, "" + (_levelGuesses[_level] - _round + 1));
        if (_currentGuess > 0)
            _sprites[SPRITE_CURRENT_GUESS].setText(c, "" + _currentGuess);
        else
            _sprites[SPRITE_CURRENT_GUESS].setText(c, "none");
    	_sprites[SPRITE_LAST_GUESS].setText(c, "" + _lastGuess);
        if ( (_round > _levelGuesses[_level]) && (_lastGuess != _room) )
            _sprites[SPRITE_LAST_DIRECTION].setText(c, _lostGuess);
        else if (_lastGuess < _room)
            _sprites[SPRITE_LAST_DIRECTION].setText(c, _lowGuess);
        else if (_lastGuess > _room)
            _sprites[SPRITE_LAST_DIRECTION].setText(c, _highGuess);
        else
            _sprites[SPRITE_LAST_DIRECTION].setText(c, _correctGuess);
    }
    

// ***********************************
// ***** METHODS -  From Display *****
// ***********************************

    /**
     * Initializes the game (using private helper methods) by getting necessary
     * resources and setting up configurable variables.
     *
     * @param dh the controlling applet/application
    */
    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);
        _random = new java.util.Random();
        dh.displayEvent(this, "LOADING 20");
        _res.addImage(BACKDROP_IMAGE, _config.getStringProperty(BACKDROP_IMAGE, "backdrop.jpg"));
        _background = _res.getImage(BACKDROP_IMAGE);
        dh.displayEvent(this, "LOADING 40");
        prepareSprites();
        dh.displayEvent(this, "LOADING 60");
        prepareSounds();
        preparePlayfield();
        dh.displayEvent(this, "LOADING 80");
        prepareLevels();
        dh.displayEvent(this, "LOADING 100");

        return true;
    }

    /**
     * Handles mouse movement. We may need to update the current guess variable
     * and if the game is over, the quit button needs messages.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
     * @param b true if mouse button down
    */
    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = _scrollbar.mouse_move(x, y, b);
        if ((b == false) || (_makingGuess))
        {
            if (_hallPlacement.contains(x,y))
            {
                _currentGuess = (_hallViewport.x + x - _hallPlacement.x) / _tileSize.width + 1;
                updateLabels();
                repaint = true;
            }
            else if (_currentGuess != 0)
            {
                _currentGuess = 0;
                updateLabels();
                repaint = true;
            }
        }
        
        if (_gameOver)
        {
            repaint |= _quitButton.mouse_move(x, y, b);
        }
        
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }
    
    /**
     * Handles the mouse being clicked. If over the hallway (playfield) then
     * player may be starting to make a guess. If the game is over then we
     * have to pass messages to the quit button.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
    */
    public void mouse_down(int x, int y)
    {
        boolean repaint = _scrollbar.mouse_down(x, y);
        _makingGuess = _hallPlacement.contains(x,y);
        
        repaint |= _soundToggle.mouse_down(x, y);
        if (_gameOver)
        {
            repaint |= _quitButton.mouse_down(x, y);
        }

        if (repaint)
            _displayHandler.refreshDisplay(this);
    }
    
    /**
     * Handles the mouse button being released. This sends messages to the
     * scrollbar and (if game over) quit button components as well as handling
     * clicking on a door in the hallway playfield.
     *
     * @param x X coordinate of mouse pointer
     * @param y Y coordinate of mouse pointer
    */
    public void mouse_up(int x, int y)
    {
        boolean repaint = _scrollbar.mouse_up(x, y);
        repaint |= _soundToggle.mouse_up(x, y);
        if ( (_makingGuess) && (_hallPlacement.contains(x,y)) && (_round <= _levelGuesses[_level]) )
        {
            int guess = (_hallViewport.x + x - _hallPlacement.x) / _tileSize.width;
            if (_hall.getAttribute(guess, 0) == 0)
            {
                Rectangle bounds = new Rectangle(_levelHistory[_level]);
                bounds.translate(_scrollbarRect.x, _scrollbarRect.y);
                bounds.translate(_scrollbarRect.width / _levelNumDoors[_level] * guess, 0);
                _historySprites[_round - 1].setBounds(bounds);
                _historySprites[_round - 1].setVisible(true);
                _lastGuess = guess + 1;
                ++_round;
                _hall.setTile(guess, 0, 0, 1);
                playSound(DOOR_SOUND);
                updateLabels();
                if ( (_lastGuess == _room) || (_round > _levelGuesses[_level]) )
                {
                    _gameOver = true;
                    if (_lastGuess == _room)
                        playSound(WIN_SOUND);
                    else
                        playSound(LOSE_SOUND);
                }
            }
        }
    
        if (_gameOver)
        {
            if (_quitButton.mouse_up(x, y))
            {
                _displayHandler.displayEvent(this, "DOORS_TITLE");
            }
        }

        _makingGuess = false;
        if (repaint)
            _displayHandler.refreshDisplay(this);
    }
    
    /**
     * Ticks occur at a specified interval (100ms in my case).
     *
     * @param ms milliseconds between this click and the last one
    */
    public void tick(int ms)
    {
        ++_tickCount;
        boolean repaint = _scrollbar.tick(ms);
        int nd = _levelNumDoors[_level];
        int ha;
        for (int cntr = 0; cntr < nd; ++cntr)
        {
            ha = _hall.getAttribute(cntr, 0);
            if ( (ha > 0) && (ha < 7) )
            {
                _hall.setTile(cntr, 0, 0, ha+1);
                repaint = true;
            }
        }
        
        if (_gameOver)
        {
            int n = (_tickCount % 4) / 2;
            if (n != _friend.getFrame())
            {
                _friend.setFrame( n );
                repaint = true;
            }
        }
        
        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

    /**
     * draws the display screen to the indicated graphics context.
     *
     * @param g Graphics context to draw to
     * @param comp Component for awt calls
    */
    public synchronized void render(Graphics g, Component comp)
    {
        if (_background == null)
        {
            Dimension d = comp.getSize();
            g.setColor(Color.white);
            g.fillRect(0,0, d.width, d.height);
        }
        else
        {
            g.drawImage(_background, 0, 0, Color.black, comp);
        }
        
        int cntr;
        
        for (cntr = 0; cntr < _sprites.length; ++cntr)
            _sprites[cntr].render(g, comp);

        _hallViewport.x = _scrollbar.getPosition();
        _hall.render(g, comp, _hallPlacement.x, _hallPlacement.y, _hallViewport);
        _scrollbar.render(g, comp);
        for (cntr = 0; cntr < _round; ++cntr)
        {
            _historySprites[cntr].render(g, comp);
        }

        if (_useSound)
        {
            _soundToggle.render(g, comp);
        }
        
        if (_gameOver) 
        {
            int ha = _hall.getAttribute(_room - 1, 0);
            if ((ha == 0) || (ha > 6))
            {
                _quitButton.render(g, comp);
                _friend.move(_hallPlacement.x + _friendAdjust.x + (_tileSize.width * (_room - 1)) - _hallViewport.x,
                        _hallPlacement.y + _friendAdjust.y);
                _friend.render(g, comp);
            }
        }
    }
    
    
    /**
     * If sound is enabled gets the resource manager to play the indicated sound.
     *
     * @param key sound to play
    */
    private void playSound(String key)
    {
        if ( (_useSound) && (_soundToggle.getState() == 1) )
        {
            _res.playAudio(key);
        }
    }

}
