package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;

// Spelchan Imports
import com.blazinggames.urcl.ui.playfield.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * This is useful for animated tiles and in cases where the tile is used as
 * part of an image (background tiles for instance). The attribute determins
 * the image that is shown with the frames being stored in a l-r,t-b grid.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class ImageStripTile
extends TileRenderer
{
    // *********************
    // ***** VARIABLES *****
    // *********************

    protected Image _imagestrip;


    // ************************
    // ***** CONSTRUCTORs *****
    // ************************

    public ImageStripTile()
    {
        super();
        _imagestrip = null;
    }

    public ImageStripTile(Image i, int w, int h)
    {
        super(w, h);
        _imagestrip = i;
    }


    // *******************************
    // ***** METHODS - RENDERING *****
    // *******************************

    public void render(Graphics g, Component c, int x, int y, int state)
    {
        int cols = Math.max(_imagestrip.getWidth(c) / Math.max(1, _width), 1);

        int sxoff = (state % cols) * _width;
        int syoff = (state / cols) * _height;

        g.drawImage(_imagestrip, x, y, x+_width, y+_height,
                    sxoff, syoff, sxoff+_width, syoff+_height, _backfill, c);
    }

}
