package com.blazinggames.retro.modern.search;

// Blazing Games imports
import com.blazinggames.urcl.URCLApplet;
import com.blazinggames.urcl.ui.Display;

// Retro imports
import com.blazinggames.retro.modern.Title;

/**
 * Ultimate Retro Project Episode 2: Modern Doors
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The applet that calls the Modern Doors game.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class ModernDoorsApplet
extends URCLApplet
{
    // *********************
    // ***** CONSTANTS *****
    // *********************

    private static final String VERSION = "1.10.031110";

    // *********************
    // ***** VARIABLES *****
    // *********************

    DoorsDisplay _game;


    // *****************************************
    // ***** METHODS - From SpelchanApplet *****
    // *****************************************

    /**
        * Prepares all the displays used by the applet. In this case, this
     * consists of the default error screen (handled by the super class),
     * the title screen, and the game display.
     */
    protected void initDisplays()
    {
        System.out.println("Modern Doors Version " + VERSION);
        System.out.println("Copyright (c)  2001,2003 Blazing Games Inc. All Rights Reserved.");
        _titleDisplay = new Title("02");
        _titleDisplay.initialize(this);
        _game = new DoorsDisplay();
        _game.initialize(this);
    }

    /**
        * @return my copyright string
     */
    public String getAppletInfo()
    {
        return "Modern DOORS Copyright (c) 2001,2003 Blazing Games Inc. All rights reserved.";
    }

    /**
        * Handles messages from the various display classes. Messages that we
     * take care of are as follows:
     * <UL>
     * <LI><B>LOADING #</B> Updates loaded %. # being percent loaded.</LI>
     * <LI><B>DOORS_LEVEL #</B>Starts doors game at level # and switches
     * to the doors display screen.</LI>
     * <LI><B>DOORS_TITLE</B>Switches to the doors title screen</LI>
     * </UL>
     *
     * @param caller The display class that is making the call.
     * @param command The command that is to be executed.
     */
    public void displayEvent(Display caller, String command)
    {
        super.displayEvent(caller, command);

        if (command.startsWith("DOORS_LEVEL"))
        {
            String s = command.substring(command.lastIndexOf(" ")+1);
            int n = Integer.parseInt(s);
            _game.setLevel(n);
            showDisplay(2);
        }
        else if (command.startsWith("DOORS_TITLE"))
        {
            showDisplay(1);
        }
    }

    /**
        * Handles switching the game screen to the indicated display ID.
     * Currently they are set up as follows:
     * <UL>
     * <LI>0 - Error screen</LI>
     * <LI>1 - Title Screen</LI>
     * <LI>2 - Doors Game Screen</LI>
     * </LI>
     *
     * @param id ID of display to switch to
     * @return the indicated display
     */
    protected Display getDisplay(int id)
    {
        if (id == 2)
            return _game;
        else
            return super.getDisplay(id);
    }

    public String getName()
    {
        return "Modern Doors";
    }
}
