package com.blazinggames.urcl.ui;

// Java imports
import java.awt.Container;

// Blazing Games imports
import com.blazinggames.urcl.data.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The display handler interface.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public interface DisplayHandler
{
	/**
	 * The resource manager should be used for handling all resources. This
	 * returns the resource manager created by the display handler.
	 * 
	 * @return resource manager
	 */
    public ResMan getResMan();
    
    /**
     * returns the configruation class that should filled with values passed
     * to the program.
     * 
     * @return config class
     */
    public Config getConfig();
    
    /**
     * Returns the container used for drawing the display
     * 
     * @return container
     */
    public Container getContainer();
    
    /**
     * used so displays can request handler to update them
     * 
     * @param caller display that is asking to be refreshed.
     */
    public void refreshDisplay(Display caller);
    
    /**
     * used so displays can call on special handler command (such as changing
     * to a different display)
     * 
     * @param caller display requesting command
     * @param command the command being sent
     */
    public void displayEvent(Display caller, String command);
    
    /**
     * logs indicated string (how is up to handler)
     * 
     * @param s string to be logged
     */
    public void log(String s);
    
    /**
     * Logs indicated string and exception (how is up to handler)
     * 
     * @param t exception to be logged
     * @param s string to be logged
     */
    public void log(Throwable t, String s);
    
    /**
     * Your basic assertion command for debugging.
     * 
     * @param b boolean expression that should be true
     * @param s description of assertion
     */
    public void assert(boolean b, String s);
}
