package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;

// Blazing Games imports
import com.blazinggames.urcl.ui.sprite.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class SScrollBar
extends Comp
{
// *********************
// ***** VARIABLES *****
// *********************
    
    protected Image _barImage;
    protected boolean _isHorizontal;
    protected int _lowRange;
    protected int _highRange;
    protected int _sliderRange;
    
    protected Slider _slider;
    protected Point _sliderOffset;
    protected int _currentLowSlider;
    protected int _sliderHoldPosition;
    
    protected boolean _clickInComponent;
    protected boolean _clickInSlider;
    

// ************************
// ***** CONSTRUCTORs *****
// ************************

    public SScrollBar()
    {
        this(0,0, 100,20, true);
    }

    public SScrollBar(int x, int y, int w, int h, boolean b)
    {
        super();
        setBounds(x, y, w, h);
        _isHorizontal = b;
        
        setSliderType(new Slider(), 0,0);
        setRange(0,255, 26);
        setSlider(0);
    }
    
// *****************************************************
// ***** METHODS - Information getting and setting *****
// *****************************************************

    public void setBarImage(Image i)
    {
        _barImage = i;
    }
    
    /**
     * Sets the orientation (horizontal scrolling or vertical scrolling).
     *
     * @param horizontal set to true if this is going to be a horizontal bar
    */
    public void setOrientation(boolean horizontal)
    {
        _isHorizontal = horizontal;
    }
    
    public boolean isHorizontal()
    {
        return _isHorizontal;
    }


// *************************************
// ***** METHODS - Slider Controls *****
// *************************************

    public void setRange(int low, int high, int bar)
    {
        _lowRange = low;
        _highRange = high;
        _sliderRange = bar;
        setSliderSize(bar);
        _changed = true;
    }
    
    public void setSliderType(Slider sl, int offsetX, int offsetY)
    {
        _slider = sl;
        _sliderOffset = new Point(offsetX, offsetY);
        setSliderSize(_sliderRange);
        _changed = true;
    }

    /**
     * Sets how high/wide the slider is.
     *
     * @param n size of bar, in same units range is in
    */
    public void setSliderSize(int n)
    {
        int rng = Math.max(1, _highRange - _lowRange);
        
        _sliderRange = n;
        if (_isHorizontal)
        {
            _slider.scale((n * _bounds.width) / rng, _slider.getHeight());
        }
        else
        {
            _slider.scale(_slider.getWidth(), (n * _bounds.height) / rng);
        }
        _changed = true;
    }
    
    /**
     * Sets the slider position within the range
     *
     * @param pos position of slider within scroll bar's range.
    */
    public void setSlider(int pos)
    {
        int n = Math.max(pos, _lowRange);
        n = Math.min(n, _highRange - _sliderRange);
        _currentLowSlider = n;
        _changed = true;
    }

    /**
     * @return the position of the left edge of the sliding bar in range
    */
    public int getPosition()
    {
        return _currentLowSlider;
    }
    
// *******************************
// ***** METHODS - From Comp *****
// *******************************

    /**
     * Called when mouse is moved. 
     *
     * @param x x coordinate of mouse pointer
     * @param y y coordinate of mouse pointer
     * @param b this is true if mouse button currently is down
     * @return true if mouse is over component.
    */
    public boolean mouse_move(int x, int y, boolean b)
    {
        if ( (b) && (_clickInSlider) )
        {
            int clickpos;
            if (_isHorizontal)
            {
                clickpos = ( ( (x - _bounds.x) * (_highRange - _lowRange)) /  _bounds.width) - _lowRange;
            }
            else
            {
                clickpos = ( ( (y - _bounds.y) * (_highRange - _lowRange)) /  _bounds.height) - _lowRange;
            }
            setSlider(clickpos - _sliderHoldPosition);
            return true;
        }
        
        if (_isActive)
            return _bounds.contains(x,y);
        else
            return false;
    }
    
    /**
     * Called when mouse has been pressed.
     * 
     * @param x x coordinate of where mouse was clicked
     * @param y y coordinate of where mouse was clicked
     * @return true if mouse went down within this component
    */
    public boolean mouse_down(int x, int y)
    {
        if (_isActive)
        {
            _isSelected = _bounds.contains(x, y);
            if (_isSelected)
            {
                int clickpos;
                if (_isHorizontal)
                {
                    clickpos = ( ( (x - _bounds.x) * (_highRange - _lowRange)) /  _bounds.width) - _lowRange;
                }
                else
                {
                    clickpos = ( ( (y - _bounds.y) * (_highRange - _lowRange)) /  _bounds.height) - _lowRange;
                }
                _clickInSlider = ((clickpos >= _currentLowSlider) && (clickpos < (_currentLowSlider + _sliderRange)));
                if (_clickInSlider)
                {
                    _slider.dragging();
                    _sliderHoldPosition = clickpos - _currentLowSlider;
                }
                _clickInComponent = !_clickInSlider;
            }
            return _isSelected;
        }
        else
            return false;
    }
    
    /** 
     * Called when mouse button has been released.
     *
     * @param x x coordinate of where mouse was released
     * @param y y coordinate of where mouse was released
     * @return true if component clicked (pressed and released in component)
    */
    public boolean mouse_up(int x, int y)
    {
        if (_isActive == false)
            return false;
        _slider.normal();
        _changed = _isSelected;
        _isSelected = false;
        if (_changed)
        {
            int clickpos;
            if (_isHorizontal)
            {
                clickpos = ( ( (x - _bounds.x) * (_highRange - _lowRange)) /  _bounds.width) - _lowRange;
            }
            else
            {
                clickpos = ( ( (y - _bounds.y) * (_highRange - _lowRange)) /  _bounds.height) - _lowRange;
            }

            if (_clickInComponent)
            {
                setSlider(clickpos);
            }
            else if (_clickInSlider)
            {
                setSlider(clickpos - _sliderHoldPosition);
            }
            _clickInComponent = false;
            _clickInSlider = false;
            return _bounds.contains(x, y);
        }
        else
            return false;
    }

    /**
     * Draws the component.
     *
     * @param g Graphics Context to draw into
     * @param c Component class that some awt commands need
    */
    public void render(Graphics g, Component c)
    {
        _changed = false;
        if (_barImage == null)
        {
            g.setColor(Color.gray);
            g.fillRect(_bounds.x, _bounds.y, _bounds.width, _bounds.height);
        }
        else
        {
            g.drawImage(_barImage, _bounds.x, _bounds.y, _bounds.width, _bounds.height, c);
        }
        
        int n = _highRange - _lowRange;
        int pos;
        if (_isHorizontal)
        {
            pos = ( (_currentLowSlider - _lowRange) * _bounds.width) / n;
            _slider.move(_bounds.x + pos + _sliderOffset.x, _bounds.y + _sliderOffset.y);
        }
        else
        {
            pos = ( (_currentLowSlider - _lowRange) * _bounds.height) / n;
            _slider.move(_bounds.x + _sliderOffset.x, _bounds.y + pos + _sliderOffset.y);
        }
        _slider.render(g, c);
    }

}
