package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.sprite.Sprite;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * The slider portion of a scroll bar. This class is used with the scrollbar
 * class.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Slider
extends Sprite
{
    // *********************
    // ***** CONSTANTS *****
    // *********************

    public static final int MODE_NORMAL = 0;
    public static final int MODE_DRAGGED = 1;

    // *********************
    // ***** VARIABLES *****
    // *********************

    protected int _mode;

    protected int _thickness;
    protected Color _color;
    protected Color _dragColor;


    // ************************
    // ***** CONSTRUCTORs *****
    // ************************

    /**
        * Standard (16x16) slider
     */
    public Slider()
    {
        this(16, 16);
    }

    public Slider(int width, int height)
    {
        super();
        scale(width, height);
        setVisible(true);
        _thickness = 2;
        _color = Color.blue;
        _dragColor = Color.green;
    }

    // *****************************************************
    // ***** METHODS - Information setting and getting *****
    // *****************************************************


    public void setThickness(int n)
    {
        _thickness = n;
    }

    public int getThickness()
    {
        return _thickness;
    }

    public void setColor(Color c)
    {
        _color = c;
    }

    public void setDragColor(Color c)
    {
        _dragColor = c;
    }

    public void setMode(int n)
    {
        _mode = n;
    }

    public void normal()
    {
        _mode = MODE_NORMAL;
    }

    public void dragging()
    {
        _mode = MODE_DRAGGED;
    }

    public int getMode()
    {
        return _mode;
    }


    // *********************************
    // ***** METHODS - From Sprite *****
    // *********************************
    /**
        * Draws the sprite on the provided graphics context. It should be noted that sprites ARE NOT required
     * to store the state of any graphics settings so it is possible that the graphics settings will
     * change. Sprite managers should store any settings for the graphics context before rendering the
     * sprites and restore them afterwards. Overriders may want to call this to automatically handle
     * drawing the background and bounding box
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
     */
    public void render(Graphics g, Component c)
    {
        render(g, c, Sprite.NO_OFFSET);
    }

    /**
        * Same as render, but adjusts the sprites coordinates by adding offset to coordinates.
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
     * @param offset point relative to top left that sprite is to be drawn
     */
    public void render(Graphics g, Component c, Point offset)
    {
        int cntr;

        if (_mode == MODE_NORMAL)
        {
            g.setColor(_color);
        }
        else
        {
            g.setColor(_dragColor);
        }

        if (_isSolid)
        {
            boolean raised = (_mode == MODE_NORMAL);
            g.fillRect(_bounds.x + offset.x, _bounds.y + offset.y, _bounds.width, _bounds.height);
            for (cntr = 0; cntr < _thickness; ++cntr)
            {
                g.draw3DRect(_bounds.x + offset.x + cntr, _bounds.y + offset.y + cntr,
                             _bounds.width - cntr - cntr, _bounds.height - cntr -cntr, raised);
            }
        }
        else
        {
            for (cntr = 0; cntr < _thickness; ++cntr)
            {
                g.drawRect(_bounds.x + offset.x + cntr, _bounds.y + offset.y + cntr,
                           _bounds.width - cntr-cntr, _bounds.height - cntr-cntr);
            }
        }
    }
}
